package at.ac.uibk.gitsearch.service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.models.WikiPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import at.ac.uibk.gitsearch.repository.gitlab.GitLabRepository;
import at.ac.uibk.gitsearch.service.dto.EditorialPageDTO;

/**
 * this service enables the provisioning of editorial pages.
 * The basic idea is, to use the wiki of a dedicated git project to edit pages.
 * From there the content is served.
 * @author Michael Breu
 *
 */
@Service
@CacheConfig(cacheNames={EditorialPagesService.PAGES_CACHE})
public class EditorialPagesService {
	
	public static final String PAGES_CACHE = "pages";
	
	private static final String editorialProject = "infrastructure/GitSearchEditorialPages";
	@Autowired
	private GitLabRepository gitLabRepository;
	
	@SuppressWarnings("deprecation")
	@Cacheable(PAGES_CACHE)
	public EditorialPageDTO getContent(String path) throws GitLabApiException {
		String encodedPath;
		try {
			encodedPath = URLEncoder.encode(path, "UTF-8");
		} catch (UnsupportedEncodingException e) {
			encodedPath = URLEncoder.encode(path);
		}
		final WikiPage page = gitLabRepository.getGitLabApi().getWikisApi().getPage(editorialProject, encodedPath);
		return new EditorialPageDTO(path, page.getContent());
	}
	
	@CacheEvict(value = PAGES_CACHE, allEntries = true)
	public boolean resetCache() {
		return true; // nothing to do
	}
	
	@Scheduled(fixedRate = 30*60*1000L) // every 30 Minutes
	public void evictPagesCacheAtIntervals() {
		resetCache();
	}

	public String getAttachmentURL(String path) throws GitLabApiException {
		return gitLabRepository.getRepositoryURL() + "infrastructure/gitsearcheditorialpages/-/wikis/" + path;
	}
	

}
