package at.ac.uibk.gitsearch.service;

import static at.ac.uibk.gitsearch.web.rest.AccountResourceIT.TEST_USER_LOGIN;

import java.io.IOException;
import java.io.InputStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import org.codeability.sharing.plugins.api.ShoppingBasket;
import org.codeability.sharing.plugins.api.search.SearchInputDTO;
import org.codeability.sharing.plugins.api.search.SearchInputMetadataDTO;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.elasticsearch.node.NodeValidationException;
import org.junit.Assert;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestServerConfiguration;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketInfoDTO;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketRedirectInfoDTO;

@SpringBootTest(classes = GitsearchApp.class)
@WithMockUser(value = TEST_USER_LOGIN, authorities = "sharing")
public class ShoppingBasketServiceIT {
	
	private static final String TEST_ZIP_LOCATION = "./testData/junit-quality-tests-exercise-master.zip";
	@Autowired
	protected ShoppingBasketService shoppingBasketService;
	@Autowired
	private SearchService searchService;

	
	private static final Logger log = LoggerFactory.getLogger(ShoppingBasketServiceIT.class);

	@BeforeAll
	public static void setUpESServer() throws IOException, NodeValidationException {
		ElasticSearchTestServerConfiguration.getTestInstance().startTestNode();
	}

    @Test
	public void testRepackage() throws IOException {
		
		final InputStream resourceStream = this.getClass().getResourceAsStream(TEST_ZIP_LOCATION);

		Assert.assertNotNull("cannot find resource at " + TEST_ZIP_LOCATION, resourceStream);
		final ZipInputStream testZip = new ZipInputStream(resourceStream);
		
		final InputStream testRepackagedZipIS = shoppingBasketService.rePackageGitLabProjectZip(testZip, "from " + TEST_ZIP_LOCATION);

		final ZipInputStream testRepackagedZip = new ZipInputStream(testRepackagedZipIS);
		
		ZipEntry entry = testRepackagedZip.getNextEntry();
		
		while(entry!=null) {
			log.info("found entry {} (directory: {})", entry.getName(), entry.isDirectory());
			Assert.assertTrue("original folder not chopped off?", !entry.getName().startsWith("junit-quality-tests-exercise-master"));
			entry = testRepackagedZip.getNextEntry();
		}
		
		testRepackagedZip.close();
	}
    
    @Test
    public void testGetBasket() throws IOException {
    	
    	// just load everything from index :-)
		final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, null,null);
		SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
		SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery);
		Assert.assertTrue("At least one hit?", searchResultPage.getSearchResult().size() >= 1);
    	
    	ShoppingBasketInfoDTO sbi = new ShoppingBasketInfoDTO(searchResultPage.getSearchResult().toArray(new SearchResultDTO[] {}));
    	
    	ShoppingBasketRedirectInfoDTO redInfo = shoppingBasketService.getRedirectInfo(sbi, "http://unused/unused");
    	
    	Pattern tokenParser = Pattern.compile("/([^/?]*)\\?");
    	final Matcher matcher = tokenParser.matcher(redInfo.getRedirectURL());
    	Assert.assertTrue(matcher.find());
    	String token=matcher.group(1);
    	final ShoppingBasket basket = shoppingBasketService.getBasket(token);
    	
    	int count=0;
    	for(SearchResultDTO hitResult: searchResultPage.getSearchResult()) {
    		// we assume same sequence
    		Assert.assertEquals(basket.exerciseInfo[count].getMetadata().getTitle(), hitResult.getMetadata().getTitle());
    		count++;
    	}
    }
	

}
